<?php
/*******************************************************************************
 * Sauce Software
 ******************************************************************************* 
 * @copyright Copyright (C) 2006 Sauce Software Pty Ltd 
 * @version $Id: vtiger_invoices.php,v 1.6 2006/06/14 04:25:18 andrew Exp $
 ******************************************************************************* 
 * @Author: Andrew Patterson  (andrew.patterson@saucesoft.com)
 * Example SOAP service that can be used to create and maintain invoices
 * 
 * Change Log:
 * Andrew Patterson   21/06/2006      Original creation 
 ******************************************************************************/
 
chdir('../');
require_once("config.php");
require_once('modules/Contacts/Contact.php');
require_once('modules/Products/Product.php');
require_once('modules/HelpDesk/HelpDesk.php');
require_once('include/logging.php');
require_once('include/database/PearDatabase.php');
require_once('include/nusoap/nusoap.php');

$NAMESPACE = 'http://www.vtigercrm.com/vtigercrm';
$server = new soap_server;

$server->configureWSDL('vtigersoap', 'urn:vtigersoap');

/* -------------------------- Data Type Definitions ------------------------- */

$server->wsdl->addComplexType(
	'invoice',
	'complexType',
	'struct',
	'all',
	'',
	array( 'invoice_id' => array('name'=>'invoice_id','type'=>'xsd:string'),
	       'subject' => array('name'=>'subject','type'=>'xsd:string'),
         'total' => array('name'=>'total','type'=>'xsd:string'),
         'status' => array('name'=>'status','type'=>'xsd:string'),
         'created' => array('name'=>'created','type'=>'xsd:string') )
);

$server->wsdl->addComplexType(
	'invoiceArray',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array( array('ref'=>'SOAP-ENC:arrayType','wsdl:arrayType'=>'tns:invoice[]') ),
	'tns:invoice'
);

/* -------------------------- Function Definitions -------------------------- */

// create_invoice
$server->register('create_invoice',
                  array('subject' => 'xsd:string',
                        'account_id' => 'xsd:string',
                        'invoice_status' => 'xsd:string'),
                  array('return' => 'xsd:string'),
                  $NAMESPACE);

/**
 * Create a new Invoice
 *
 * PRE-COND: $subject is not empty
 *           $account_id is not empty and is a valid vTiger account
 *           all other data elements (if supplied) are valid types and values
 *           DATES must be formatted as 'yyyy-mm-dd'
 * POST-COND: a new invoice is created
 *
 * @param string $subject
 * @param int $account_id - must be a valid vTiger account
 * @param string $invoice_status - Defaults to 'Created'
 * @return integer - new invoice id, or '0' on error
 */
function create_invoice($subject, $account_id, $invoice_status = 'Created') {

    // check for mandatory values
    if (empty($subject) || empty($account_id)) {
    	return 0;
    }

    global $adb;

    // use the existing module to manage invoices
    require_once('modules/Invoice/Invoice.php');

    $my_invoice = new Invoice();

    // save() expects dates to be in user display format
    $now = getDisplayDate(date('Y-m-d'));

    // populate the given data for the new product
    // don't give it an ID - this will be auto generated
    $my_invoice->column_fields[subject] = $subject;
    $my_invoice->column_fields[txtTax] = 0;
    $my_invoice->column_fields[hdnSubTotal] = 0;
    $my_invoice->column_fields[hdnGrandTotal] = 0;
    $my_invoice->column_fields[account_id] = $account_id;
    $my_invoice->column_fields[invoicestatus] = $invoice_status;
    $my_invoice->column_fields[invoicedate] = $now;
    $my_invoice->column_fields[duedate] = $now;

    @$my_invoice->save('Invoice');

    return $my_invoice->id;
}

// add_product_to_invoice
$server->register('add_product_to_invoice',
                  array('invoice_id' => 'xsd:string',
                        'product_id' => 'xsd:string',
                        'quantity' => 'xsd:string',
                        'item_price' => 'xsd:string'),
                  array('return' => 'xsd:string'),
                  $NAMESPACE);

/**
 * Add a Product to an Invoice
 *
 * PRE-COND: $invoice_id is not empty and is a valid vTiger invoice
 *           $product_id is not empty and is a valid vTiger product
 *           all other data elements (if supplied) are valid types and values
 * POST-COND: a new product is added to an existing invoice
 *
 * @param int $invoice_id
 * @param int $product_id
 * @param int $quantity
 * @param float $item_price 
 * @return boolean - true if created, false otherwise
 */
function add_product_to_invoice($invoice_id, $product_id, $quantity = 1, $item_price = 0) {

    // check for mandatory values
    if (empty($invoice_id) || empty($product_id)) {
    	return false;
    }

    global $adb;

    // Add product to invoice
    $query_string ="INSERT INTO invoiceproductrel (invoiceid, productid, quantity, listprice) VALUES (".$invoice_id.",".$product_id.",".$quantity.",".$item_price.")";
    $adb->query($query_string);
    
    // Update total amount in invoice
    $product_total = $quantity * $item_price;
    $query_string = "UPDATE invoice SET subtotal = subtotal + ".$product_total.", total = total + ".$product_total." WHERE invoiceid = ".$invoice_id;
    $adb->query($query_string);
    
    // Deduct product from total product quantity
	  $query= "UPDATE products SET qtyinstock = qtyinstock - ".$quantity." WHERE productid = ".$product_id;
    $adb->query($query);

    return true;
}

// remove_product_from_invoice
$server->register('remove_product_from_invoice',
                  array('invoice_id' => 'xsd:string',
                        'product_id' => 'xsd:string'),
                  array('return' => 'xsd:string'),
                  $NAMESPACE);
/**
 * Remove a Product to an Invoice
 *
 * PRE-COND: $invoice_id is not empty and is a valid vTiger invoice
 *           $product_id is not empty and is a valid vTiger product
 * POST-COND: the specified product is removed from the invoice
 *
 * @param int $invoice_id
 * @param int $product_id
 * @return boolean - true if removed, false otherwise
 */
function remove_product_from_invoice($invoice_id, $product_id) {

    // check for mandatory values
    if (empty($invoice_id) || empty($product_id)) {
    	return false;
    }

    global $adb;
    
    // Get the quantity of the product in the order
    $query_string = "SELECT quantity, listprice FROM invoiceproductrel WHERE invoiceid = ".$invoice_id." AND productid = ".$product_id;
    $result = $adb->query($query_string);
    $quantity = $adb->query_result($result, 0, "quantity");
    $item_price = $adb->query_result($result, 0, "listprice");

    // Remove product from the invoice
    $query_string ="DELETE FROM invoiceproductrel WHERE invoiceid = ".$invoice_id." AND productid = ".$product_id;
    $adb->query($query_string);
    
    // Update total amount in invoice
    $product_total = $quantity * $item_price;
    $query_string = "UPDATE invoice SET subtotal = subtotal - ".$product_total.", total = total - ".$product_total." WHERE invoiceid = ".$invoice_id;
    $adb->query($query_string);
    
    // Add products to the current quantity
	  $query= "UPDATE products SET qtyinstock = qtyinstock + ".$quantity." WHERE productid = ".$product_id;
    $adb->query($query);

    return true;
}

// add_tax
$server->register('add_tax',
                  array('invoice_id' => 'xsd:string',
                        'amount' => 'xsd:string'),
                  array('return' => 'xsd:string'),
                  $NAMESPACE);
/**
 * Add amount to tax total on an invoice
 *
 * PRE-COND: $invoice_id is not empty and is a valid vTiger invoice
 * POST-COND: the specified amount is added to the tax total on an invoice
 *
 * @param int $invoice_id
 * @param int $amount
 * @return boolean - true if added, false otherwise
 */
function add_tax($invoice_id, $amount) {

    // check for mandatory values
    if (empty($invoice_id) || empty($amount)) {
    	return false;
    }

    global $adb;
    
    // Update tax amount in invoice
    $query_string = "UPDATE invoice SET salestax = salestax + ".$amount.", total = total + ".$amount." WHERE invoiceid = ".$invoice_id;
    $adb->query($query_string);
    
    return true;
}

// remove_tax
$server->register('remove_tax',
                  array('invoice_id' => 'xsd:string',
                        'amount' => 'xsd:string'),
                  array('return' => 'xsd:string'),
                  $NAMESPACE);
/**
 * Remove an amount from the total tax on an invoice
 *
 * PRE-COND: $invoice_id is not empty and is a valid vTiger invoice
 * POST-COND: the specified amount is removed from the invoice tax total
 *
 * @param int $invoice_id
 * @param int $amount
 * @return boolean - true if removed, otherwise false
 */
function remove_tax($invoice_id, $amount) {

    // check for mandatory values
    if (empty($invoice_id) || empty($amount)) {
    	return false;
    }

    global $adb;
    
    // Update tax amount in invoice
    $query_string = "UPDATE invoice SET salestax = salestax - ".$amount.", total = total + ".$amount." WHERE invoiceid = ".$invoice_id;
    $adb->query($query_string);
    
    return true;
}

// remove_tax
$server->register('add_adjustment',
                  array('invoice_id' => 'xsd:string',
                        'amount' => 'xsd:string'),
                  array('return' => 'xsd:string'),
                  $NAMESPACE);
/**
 * Adjusts the invoice total by the specified amount
 *
 * PRE-COND: $invoice_id is not empty and is a valid vTiger invoice
 * POST-COND: the specified amount is added to the invoice adjustment column
 *
 * @param int $invoice_id
 * @param int $amount
 * @return boolean - true if added, otherwise false
 */
function add_adjustment($invoice_id, $amount) {

    // check for mandatory values
    if (empty($invoice_id) || empty($amount)) {
    	return false;
    }

    global $adb;
    
    // Update tax amount in invoice
    $query_string = "UPDATE invoice SET adjustment = adjustment + ".$amount.", total = total + ".$amount." WHERE invoiceid = ".$invoice_id;
    $adb->query($query_string);
    
    return true;
}

// remove_tax
$server->register('set_invoice_due_date',
                  array('invoice_id' => 'xsd:string',
                        'due_date' => 'xsd:string'),
                  array('return' => 'xsd:string'),
                  $NAMESPACE);
/**
 * Sets the due date for the invoice
 *
 * PRE-COND: $invoice_id is not empty and is a valid vTiger invoice, $due_date is specified in the format Y-m-d
 * POST-COND: the specified date is set as the due date for the invoice
 *
 * @param int $invoice_id
 * @param int $due_date - must be in the format Y-m-d
 * @return boolean - true if added, otherwise false
 */
function set_invoice_due_date($invoice_id, $due_date) {

    // check for mandatory values
    if (empty($invoice_id) || empty($due_date)) {
    	return false;
    }

    global $adb;
    
    // Update tax amount in invoice
    $query_string = "UPDATE invoice SET duedate = '".$due_date."' WHERE invoiceid = ".$invoice_id;
    $adb->query($query_string);
    
    return true;
}


// remove_tax
$server->register('update_entity_description',
                  array('crmentity_id' => 'xsd:string',
                        'description' => 'xsd:string'),
                  array('return' => 'xsd:string'),
                  $NAMESPACE);
/**
 * Adds a description to an invoice
 *
 * PRE-COND: $invoice_id is not empty and is a valid vTiger invoice
 * POST-COND: none
 *
 * @param int $invoice_id
 * @param string $description
 * @return boolean - true if added, otherwise false
 */
function update_entity_description($crmentity_id, $description) {

    // check for mandatory values
    if (empty($crmentity_id) || empty($description)) {
    	return false;
    }

    global $adb;
    
    // Update tax amount in invoice
    $query_string = "UPDATE crmentity SET description = '".$description."' WHERE crmid = ".$crmentity_id;
    $adb->query($query_string);
    
    return true;
}

// add_invoice_description
$server->register('add_entity_description',
                  array('crmentity_id' => 'xsd:string',
                        'description' => 'xsd:string'),
                  array('return' => 'xsd:string'),
                  $NAMESPACE);
/**
 * Adds a description to an invoice
 *
 * PRE-COND: $invoice_id is not empty and is a valid vTiger invoice
 * POST-COND: none
 *
 * @param int $invoice_id
 * @param string $description
 * @return boolean - true if added, otherwise false
 */
function add_entity_description($crmentity_id, $description) {

    // check for mandatory values
    if (empty($crmentity_id) || empty($description)) {
    	return false;
    }

    global $adb;
    
    // Update tax amount in invoice
    $query_string = "UPDATE crmentity SET description = '".$description."' WHERE crmid = ".$crmentity_id;
    $adb->query($query_string);
    
    return true;
}

// update_invoice_status
$server->register('update_invoice_status',
                  array('invoice_id' => 'xsd:string',
                        'status' => 'xsd:string'),
                  array('return' => 'xsd:string'),
                  $NAMESPACE);

/**
 * Updates the status of an existing invoice
 *
 * PRE-COND: $invoice_id is not empty and is a valid vTiger invoice
 *           $status is not empty
 * POST-COND: The specified invoice will have an updated status
 *
 * @param int $invoice_id
 * @param string $status
 * @return integer - invoice id, or '0' on error
 */
function update_invoice_status($invoice_id, $status) {
    // check for mandatory values
    if (empty($invoice_id) || empty($status)) {
    	return 0;
    }
    
    global $adb;

    // use the existing module to manage products
    require_once('modules/Invoice/Invoice.php');

    $my_invoice = new Invoice();
    $my_invoice->id = $invoice_id;
    $my_invoice->mode = 'edit';

    // retrieve existing product data
    $my_invoice->retrieve_entity_info($invoice_id, 'Invoice');
    $my_invoice->column_fields[invoicestatus] = $status;
    $invoice_due_date = $my_invoice->column_fields[duedate];
    $invoice_date = $my_invoice->column_fields[invoicedate];
    @$my_invoice->save('Invoice');
    
    // Fix: stops due date from disappearing on invoice update
    $query_string = "UPDATE invoice SET duedate = '".$invoice_due_date."', invoicedate = '".$invoice_date."' WHERE invoiceid = '".$invoice_id."'";
    $adb->query($query_string);
    
    return true;
}


// add_purchase_order
$server->register('add_purchase_order',
                  array('invoice_id' => 'xsd:string',
                        'purchase_order' => 'xsd:string'),
                  array('return' => 'xsd:string'),
                  $NAMESPACE);
/**
 * Adds the purchase order to the invoice.
 *
 * PRE-COND: $invoice_id is not empty and is a valid vTiger invoice
 * POST-COND: the purchase order is added to the invoice.
 *
 * @param int $invoice_id
 * @param string $purchase_order
 * @return boolean - true if added, otherwise false
 */
function add_purchase_order($invoice_id, $purchase_order) {

    // check for mandatory values
    if (empty($invoice_id)) {
    	return false;
    }

    global $adb;
    
    // Update tax amount in invoice
    $query_string = "UPDATE invoice SET purchaseorder = '".$purchase_order."' WHERE invoiceid = ".$invoice_id;
    $adb->query($query_string);
    
    return true;
}


// get_invoices
$server->register('get_invoices',
                  array('account_id' => 'xsd:string',
                        'filter' => 'xsd:string',
                        'order' => 'xsd:string'),
                  array('return' => 'tns:invoiceArray'),
                  $NAMESPACE);
/**
 * Retrieves a list of invoices for a specific account.
 *
 * PRE-COND: $account_id must be a valid vTiger account if specified
 * POST-COND: the invoices for the account are returned.
 *
 * @param int $account_id
 * @param string $filter
 * @param string $order  
 * @return invoice $invoices
 */
function get_invoices($account_id, $filter, $order) {

    // check for mandatory values
    if (empty($account_id)) {
    	return false;
    }

    global $adb;
    
    // Grab the invoices.
    $query_string  = "SELECT * FROM crmentity, invoice ";
    $query_string .= "WHERE crmentity.crmid = invoice.invoiceid ";
    $query_string .= "AND crmentity.deleted = 0 ";
    
    // Use account ID filter if specified
    if ($account_id != '') {
      $query_string .= "WHERE accountid = '".addslashes($account_id)."' ";
    }

    // Use subject filter if specified
    if ($filter != '') {
      $words = explode(" ", $filter);
      $query_string .= " AND (0 ";
      foreach ($words as $word) {
        $query_string .= "OR subject LIKE '%".$word."%' ";
      }
      $query_string .= ")"; 
    } 
  
    // Set ordering
    $orders = array('invoiceid', 'rinvoiceid', 'subject', 'rsubject', 'total', 'rtotal', 'invoicedate', 'rinvoicedate', 'invoicestatus', 'rinvoicestatus');
    $by = '';
    if (!in_array($order, $orders)) {
      $order = 'rinvoiceid';
    }
  	if (substr($order, 0, 1) == 'r') {
      $order = substr($order, 1);
      $by = 'DESC';
    }
    $query_string .= ' ORDER BY '.$order.' '.$by;     

    $invoices_rt = $adb->query($query_string);
    $output_list = array();
    
    while ($invoices_rt && ($invoice = $adb->fetch_array($invoices_rt))) {
      $record = array('invoice_id' => $invoice['invoiceid'],
                      'subject' => $invoice['subject'],
                      'total' => $invoice['total'],
                      'status' => $invoice['invoicestatus'],
                      'created' => $invoice['invoicedate']);
      $output_list[] = $record;
    }
    
    return $output_list;
}

// get_invoice_by_id
$server->register('get_invoice_by_id',
                  array('invoice_id' => 'xsd:string'),
                  array('return' => 'tns:invoice'),
                  $NAMESPACE);
/**
 * Retrieves a list of invoices for a specific account.
 *
 * PRE-COND: $account_id is not empty and is a valid vTiger account
 * POST-COND: the invoices for the account are returned.
 *
 * @param int $account_id
 * @param string $filter
 * @param string $order  
 * @return invoice $invoices
 */
function get_invoice_by_id($invoice_id) {

  // check for mandatory values
  if (empty($invoice_id)) {
  	return false;
  }

  global $adb;
  
  // Get the required invoice
  $query_string = "SELECT * FROM invoice WHERE invoiceid = '".$invoice_id."'";
  $invoices_rt = $adb->query($query_string);
  
  if ($invoices_rt && $adb->num_rows($invoices_rt)) {
    $invoice = $adb->fetch_array($invoices_rt);
    $record = array('invoice_id' => $invoice['invoiceid'],
                    'subject' => $invoice['subject'],
                    'total' => round(($invoice['total'] * 100), 0), // Amount must be in cents
                    'status' => $invoice['invoicestatus'],
                    'created' => $invoice['invoicedate']);
    return $record;
  } else {
    return false;
  }
}

/* Begin the HTTP listener service and exit. */
$HTTP_RAW_POST_DATA = isset($HTTP_RAW_POST_DATA) ? $HTTP_RAW_POST_DATA : '';
$server->service($HTTP_RAW_POST_DATA);

exit();

?>
